/*
** Julian Pidancet <julian.pidancet@citrix.com>
**
** WARNING: This file is generated. Be sure you are editing rpc_definitions.m4
** or dmbus.h.in instead of dmbus.h. Otherwise your changes WILL BE LOST.
*/

#ifndef __DMBUS_H__
# define __DMBUS_H__

# include <stdint.h>
# include <libargo.h>

# ifdef __cplusplus
extern "C"
{
# endif
    /**
     * SHA1 Hash of the interface source files.
     */
# define DMBUS_SHA1_STRING "86fa45cc1f2aaddf160ec3ee784deba0599954bf"

    /**
     * dmbus services definitions
     */
# define DMBUS_BASE_PORT    4000
    enum dmbus_service_id
    {
        DMBUS_SERVICE_DUMMY = 0, /* Only for test programs */
        DMBUS_SERVICE_SURFMAN = 1,
        DMBUS_SERVICE_INPUT = 2,

        DMBUS_SERVICE_MAX,
    };

    /**
     * dmbus types definitions
     */
# define DMBUS_ID_INVALID ((uint32_t)-1)
    enum
    {
        DEVICE_TYPE_XENFB = 0,
        DEVICE_TYPE_VESA,
        DEVICE_TYPE_XENGFX,
        DEVICE_TYPE_PASSTHROUGH,
        DEVICE_TYPE_EMULATION,
        DEVICE_TYPE_INPUT,
        DEVICE_TYPE_INPUT_PVM
    };
    enum
    {
        FRAMEBUFFER_FORMAT_RGB555 = (1 << 0),
        FRAMEBUFFER_FORMAT_BGR555 = (1 << 1),
        FRAMEBUFFER_FORMAT_RGB565 = (1 << 2),
        FRAMEBUFFER_FORMAT_BGR565 = (1 << 3),
        FRAMEBUFFER_FORMAT_RGB888 = (1 << 4),
        FRAMEBUFFER_FORMAT_BGR888 = (1 << 5),
        FRAMEBUFFER_FORMAT_RGBX8888 = (1 << 6),
        FRAMEBUFFER_FORMAT_BGRX8888 = (1 << 7),
    };

    typedef uint32_t                DeviceType;
    typedef uint32_t                FramebufferFormat;
    typedef struct {
        uint8_t b[256];
    }                               Buff256;

    typedef struct {
        uint8_t slot;
        char name[26];
        uint32_t evbits;
        uint64_t bits[1];
    } InputConfig;    

    typedef void *dmbus_client_t;

    struct dmbus_rpc_ops;
    struct dmbus_service_ops
    {
        int (*connect)(dmbus_client_t client,
                       int domain,
                       DeviceType type,
                       int dm_domain,
                       int fd,
                       struct dmbus_rpc_ops **ops,
                       void **priv);
        void (*disconnect)(dmbus_client_t client, void *priv);
    };

# define DMBUS_MAX_MSG_LEN 512
# define DMBUS_PACKED __attribute__ ((packed))
    /**
     * dmbus connection prologue
     */
    struct dmbus_conn_prologue
    {
        uint32_t domain;
        DeviceType type;
        uint8_t hash[20];
    } DMBUS_PACKED;

    /**
     * dmbus message format
     */
    union dmbus_msg
    {
        struct dmbus_msg_hdr
        {
            uint32_t msg_len;
            uint32_t msg_type;
            uint32_t return_value;
        } DMBUS_PACKED hdr;

        /**
         * WARNING:
         *
         * The following section contains generated definitions.
         */

#define DMBUS_MSG_DISPLAY_RESIZE 0
struct msg_display_resize
{
    struct dmbus_msg_hdr hdr;
    uint8_t DisplayID;
    uint16_t width;
    uint16_t height;
    uint32_t linesize;
    uint64_t lfb_addr;
    uint8_t lfb_traceable;
    FramebufferFormat format;
    uint32_t fb_offset;
} DMBUS_PACKED display_resize;
#define DMBUS_MSG_DISPLAY_GET_INFO 1
struct msg_display_get_info
{
    struct dmbus_msg_hdr hdr;
    uint8_t DisplayID;
} DMBUS_PACKED display_get_info;
#define DMBUS_MSG_DISPLAY_INFO 2
struct msg_display_info
{
    struct dmbus_msg_hdr hdr;
    uint8_t DisplayID;
    uint16_t max_xres;
    uint16_t max_yres;
    uint16_t align;
} DMBUS_PACKED display_info;
#define DMBUS_MSG_EMPTY_REPLY 8
struct msg_empty_reply
{
    struct dmbus_msg_hdr hdr;
    ;
} DMBUS_PACKED empty_reply;
#define DMBUS_MSG_SWITCHER_ABS 16
struct msg_switcher_abs
{
    struct dmbus_msg_hdr hdr;
    int32_t enabled;
} DMBUS_PACKED switcher_abs;
#define DMBUS_MSG_SWITCHER_PVM_DOMID 17
struct msg_switcher_pvm_domid
{
    struct dmbus_msg_hdr hdr;
    uint32_t domid;
    int32_t slot;
} DMBUS_PACKED switcher_pvm_domid;
#define DMBUS_MSG_SWITCHER_DOMID 18
struct msg_switcher_domid
{
    struct dmbus_msg_hdr hdr;
    uint32_t domid;
    int32_t slot;
} DMBUS_PACKED switcher_domid;
#define DMBUS_MSG_SWITCHER_LEDS 19
struct msg_switcher_leds
{
    struct dmbus_msg_hdr hdr;
    int32_t led_code;
} DMBUS_PACKED switcher_leds;
#define DMBUS_MSG_SWITCHER_SHUTDOWN 20
struct msg_switcher_shutdown
{
    struct dmbus_msg_hdr hdr;
    int32_t reason;
} DMBUS_PACKED switcher_shutdown;
#define DMBUS_MSG_DOM0_INPUT_EVENT 21
struct msg_dom0_input_event
{
    struct dmbus_msg_hdr hdr;
    uint16_t type;
    uint16_t code;
    int32_t value;
} DMBUS_PACKED dom0_input_event;
#define DMBUS_MSG_INPUT_CONFIG_RESET 24
struct msg_input_config_reset
{
    struct dmbus_msg_hdr hdr;
    uint8_t slot;
} DMBUS_PACKED input_config_reset;
#define DMBUS_MSG_INPUT_CONFIG 25
struct msg_input_config
{
    struct dmbus_msg_hdr hdr;
    InputConfig c;
} DMBUS_PACKED input_config;
#define DMBUS_MSG_INPUT_WAKEUP 26
struct msg_input_wakeup
{
    struct dmbus_msg_hdr hdr;
    ;
} DMBUS_PACKED input_wakeup;
#define DMBUS_MSG_DEVICE_MODEL_READY 23
struct msg_device_model_ready
{
    struct dmbus_msg_hdr hdr;
    ;
} DMBUS_PACKED device_model_ready;
        /**
         * End of generated definitions section.
         */

    };

    struct dmbus_rpc_ops
    {
        /**
         * WARNING:
         *
         * The following section contains generated definitions.
         */
int (*display_resize)(void *priv, struct msg_display_resize *msg, size_t msglen, struct msg_empty_reply *out);
int (*display_get_info)(void *priv, struct msg_display_get_info *msg, size_t msglen, struct msg_display_info *out);
void (*switcher_abs)(void *priv, struct msg_switcher_abs *msg, size_t msglen);
void (*switcher_pvm_domid)(void *priv, struct msg_switcher_pvm_domid *msg, size_t msglen);
void (*switcher_domid)(void *priv, struct msg_switcher_domid *msg, size_t msglen);
void (*switcher_leds)(void *priv, struct msg_switcher_leds *msg, size_t msglen);
void (*switcher_shutdown)(void *priv, struct msg_switcher_shutdown *msg, size_t msglen);

        /**
         * End of generated definitions section.
         */
    };

/**
 * WARNING:
 *
 * The following section contains generated definitions.
 */

void dom0_input_event(dmbus_client_t client, struct msg_dom0_input_event *msg, size_t msglen);
void input_config(dmbus_client_t client, struct msg_input_config *msg, size_t msglen);
void input_config_reset(dmbus_client_t client, struct msg_input_config_reset *msg, size_t msglen);
void input_wakeup(dmbus_client_t client, struct msg_input_wakeup *msg, size_t msglen);
void device_model_ready(dmbus_client_t client, struct msg_device_model_ready *msg, size_t msglen);
/**
 * End of generated definitions section.
 */
void dmbus_cleanup(void);
int dmbus_init(int service_id, struct dmbus_service_ops *service_ops);
void dmbus_handle_connect(int fd);
void dmbus_handle_events(dmbus_client_t client);
void dmbus_client_disconnect(dmbus_client_t client);

#ifdef __cplusplus
}
#endif

#endif /* __DMBUS_H__ */
